@@ -1,16 +1,15 @@
 /*
  * OS/2 support routines for PhysicsFS.
  *
- * Please see the file LICENSE in the source's root directory.
+ * Please see the file LICENSE.txt in the source's root directory.
  *
  *  This file written by Ryan C. Gordon.
  */
 
-#if HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#define __PHYSICSFS_INTERNAL__
+#include "physfs_platforms.h"
 
-#if (defined OS2)
+#ifdef PHYSFS_PLATFORM_OS2
 
 #define INCL_DOSSEMAPHORES
 #define INCL_DOSDATETIME
@@ -30,7 +29,6 @@
 #include <time.h>
 #include <ctype.h>
 
-#define __PHYSICSFS_INTERNAL__
 #include "physfs_internal.h"
 
 const char *__PHYSFS_platformDirSeparator = "\\";
@@ -147,7 +145,7 @@ static void cvt_path_to_correct_case(char *buf)
         {
             while (count == 1)  /* while still entries to enumerate... */
             {
-                if (__PHYSFS_platformStricmp(fb.achName, fname) == 0)
+                if (__PHYSFS_stricmpASCII(fb.achName, fname) == 0)
                 {
                     strcpy(fname, fb.achName);
                     break;  /* there it is. Overwrite and stop searching. */
@@ -199,7 +197,7 @@ int __PHYSFS_platformInit(void)
     /* The string is capitalized! Figure out the REAL case... */
     cvt_path_to_correct_case(buf);
 
-    baseDir = (char *) malloc(len + 1);
+    baseDir = (char *) allocator.Malloc(len + 1);
     BAIL_IF_MACRO(baseDir == NULL, ERR_OUT_OF_MEMORY, 0);
     strcpy(baseDir, buf);
     return(1);  /* success. */
@@ -209,7 +207,7 @@ int __PHYSFS_platformInit(void)
 int __PHYSFS_platformDeinit(void)
 {
     assert(baseDir != NULL);
-    free(baseDir);
+    allocator.Free(baseDir);
     baseDir = NULL;
     return(1);  /* success. */
 } /* __PHYSFS_platformDeinit */
@@ -254,21 +252,14 @@ static int is_cdrom_drive(ULONG drive)
 } /* is_cdrom_drive */
 
 
-char **__PHYSFS_platformDetectAvailableCDs(void)
+void __PHYSFS_platformDetectAvailableCDs(PHYSFS_StringCallback cb, void *data)
 {
-    ULONG dummy;
-    ULONG drivemap;
+    ULONG dummy = 0;
+    ULONG drivemap = 0;
     ULONG i, bit;
-    APIRET rc;
-    char **retval;
-    PHYSFS_uint32 cd_count = 1;   /* we count the NULL entry. */
-
-    retval = (char **) malloc(sizeof (char *));
-    BAIL_IF_MACRO(retval == NULL, ERR_OUT_OF_MEMORY, NULL);
-    *retval = NULL;
-
-    rc = DosQueryCurrentDisk(&dummy, &drivemap);
-    BAIL_IF_MACRO(os2err(rc) != NO_ERROR, NULL, retval);
+    APIRET rc = DosQueryCurrentDisk(&dummy, &drivemap);
+    if (os2err(rc) != NO_ERROR)
+        return;
 
     for (i = 0, bit = 1; i < 26; i++, bit <<= 1)
     {
@@ -276,33 +267,18 @@ char **__PHYSFS_platformDetectAvailableCDs(void)
         {
             if ((is_cdrom_drive(i)) && (disc_is_inserted(i)))
             {
-                char **tmp = realloc(retval, sizeof (char *) * (cd_count + 1));
-                if (tmp)
-                {
-                    char *str = (char *) malloc(4);
-                    retval = tmp;
-                    retval[cd_count - 1] = str;
-                    if (str)
-                    {
-                        str[0] = ('A' + i);
-                        str[1] = ':';
-                        str[2] = '\\';
-                        str[3] = '\0';
-                        cd_count++;
-                    } /* if */
-                } /* if */
+                char drive[4] = "x:\\";
+                drive[0] = ('A' + i);
+                cb(data, drive);
             } /* if */
         } /* if */
     } /* for */
-
-    retval[cd_count - 1] = NULL;
-    return(retval);
 } /* __PHYSFS_platformDetectAvailableCDs */
 
 
 char *__PHYSFS_platformCalcBaseDir(const char *argv0)
 {
-    char *retval = (char *) malloc(strlen(baseDir) + 1);
+    char *retval = (char *) allocator.Malloc(strlen(baseDir) + 1);
     BAIL_IF_MACRO(retval == NULL, ERR_OUT_OF_MEMORY, NULL);
     strcpy(retval, baseDir); /* calculated at init time. */
     return(retval);
@@ -321,50 +297,6 @@ char *__PHYSFS_platformGetUserDir(void)
 } /* __PHYSFS_platformGetUserDir */
 
 
-int __PHYSFS_platformStricmp(const char *x, const char *y)
-{
-    int ux, uy;
-
-    do
-    {
-        ux = toupper((int) *x);
-        uy = toupper((int) *y);
-        if (ux > uy)
-            return(1);
-        else if (ux < uy)
-            return(-1);
-        x++;
-        y++;
-    } while ((ux) && (uy));
-
-    return(0);
-} /* __PHYSFS_platformStricmp */
-
-
-int __PHYSFS_platformStrnicmp(const char *x, const char *y, PHYSFS_uint32 len)
-{
-    int ux, uy;
-
-    if (!len)
-        return(0);
-
-    do
-    {
-        ux = toupper((int) *x);
-        uy = toupper((int) *y);
-        if (ux > uy)
-            return(1);
-        else if (ux < uy)
-            return(-1);
-        x++;
-        y++;
-        len--;
-    } while ((ux) && (uy) && (len));
-
-    return(0);
-} /* __PHYSFS_platformStrnicmp */
-
-
 int __PHYSFS_platformExists(const char *fname)
 {
     FILESTATUS3 fs;
@@ -388,6 +320,7 @@ int __PHYSFS_platformIsDirectory(const char *fname)
 } /* __PHYSFS_platformIsDirectory */
 
 
+/* !!! FIXME: can we lose the malloc here? */
 char *__PHYSFS_platformCvtToDependent(const char *prepend,
                                       const char *dirName,
                                       const char *append)
@@ -395,7 +328,7 @@ char *__PHYSFS_platformCvtToDependent(const char *prepend,
     int len = ((prepend) ? strlen(prepend) : 0) +
               ((append) ? strlen(append) : 0) +
               strlen(dirName) + 1;
-    char *retval = malloc(len);
+    char *retval = allocator.Malloc(len);
     char *p;
 
     BAIL_IF_MACRO(retval == NULL, ERR_OUT_OF_MEMORY, NULL);
@@ -417,17 +350,24 @@ char *__PHYSFS_platformCvtToDependent(const char *prepend,
 } /* __PHYSFS_platformCvtToDependent */
 
 
-LinkedStringList *__PHYSFS_platformEnumerateFiles(const char *dirname,
-                                                  int omitSymLinks)
+void __PHYSFS_platformEnumerateFiles(const char *dirname,
+                                     int omitSymLinks,
+                                     PHYSFS_EnumFilesCallback callback,
+                                     const char *origdir,
+                                     void *callbackdata)
 {
     char spec[CCHMAXPATH];
-    LinkedStringList *ret = NULL, *p = NULL;
     FILEFINDBUF3 fb;
     HDIR hdir = HDIR_CREATE;
     ULONG count = 1;
     APIRET rc;
 
-    BAIL_IF_MACRO(strlen(dirname) > sizeof (spec) - 5, ERR_BAD_FILENAME, NULL);
+    if (strlen(dirname) > sizeof (spec) - 5)
+    {
+        __PHYSFS_setError(ERR_BAD_FILENAME);
+        return;
+    } /* if */
+
     strcpy(spec, dirname);
     strcat(spec, (spec[strlen(spec) - 1] != '\\') ? "\\*.*" : "*.*");
 
@@ -435,17 +375,19 @@ LinkedStringList *__PHYSFS_platformEnumerateFiles(const char *dirname,
                       FILE_DIRECTORY | FILE_ARCHIVED |
                       FILE_READONLY | FILE_HIDDEN | FILE_SYSTEM,
                       &fb, sizeof (fb), &count, FIL_STANDARD);
-    BAIL_IF_MACRO(os2err(rc) != NO_ERROR, NULL, 0);
+
+    if (os2err(rc) != NO_ERROR)
+        return;
+
     while (count == 1)
     {
         if ((strcmp(fb.achName, ".") != 0) && (strcmp(fb.achName, "..") != 0))
-            ret = __PHYSFS_addToLinkedStringList(ret, &p, fb.achName, -1);
+            callback(callbackdata, origdir, fb.achName);
 
         DosFindNext(hdir, &fb, sizeof (fb), &count);
     } /* while */
 
     DosFindClose(hdir);
-    return(ret);
 } /* __PHYSFS_platformEnumerateFiles */
 
 
@@ -464,14 +406,14 @@ char *__PHYSFS_platformCurrentDir(void)
     /* The first call just tells us how much space we need for the string. */
     rc = DosQueryCurrentDir(currentDisk, &byte, &pathSize);
     pathSize++; /* Add space for null terminator. */
-    retval = (char *) malloc(pathSize + 3);  /* plus "x:\\" */
+    retval = (char *) allocator.Malloc(pathSize + 3);  /* plus "x:\\" */
     BAIL_IF_MACRO(retval == NULL, ERR_OUT_OF_MEMORY, NULL);
 
     /* Actually get the string this time. */
     rc = DosQueryCurrentDir(currentDisk, (PBYTE) (retval + 3), &pathSize);
     if (os2err(rc) != NO_ERROR)
     {
-        free(retval);
+        allocator.Free(retval);
         return(NULL);
     } /* if */
 
@@ -488,7 +430,7 @@ char *__PHYSFS_platformRealPath(const char *path)
     char *retval;
     APIRET rc = DosQueryPathInfo(path, FIL_QUERYFULLNAME, buf, sizeof (buf));
     BAIL_IF_MACRO(os2err(rc) != NO_ERROR, NULL, NULL);
-    retval = (char *) malloc(strlen(buf) + 1);
+    retval = (char *) allocator.Malloc(strlen(buf) + 1);
     BAIL_IF_MACRO(retval == NULL, ERR_OUT_OF_MEMORY, NULL);
     strcpy(retval, buf);
     return(retval);
@@ -662,13 +604,13 @@ int __PHYSFS_platformEOF(void *opaque)
 
 int __PHYSFS_platformFlush(void *opaque)
 {
-    return(os2err(DosResetBuffer((HFILE) opaque) == NO_ERROR));
+    return(os2err(DosResetBuffer((HFILE) opaque)) == NO_ERROR);
 } /* __PHYSFS_platformFlush */
 
 
 int __PHYSFS_platformClose(void *opaque)
 {
-    return(os2err(DosClose((HFILE) opaque) == NO_ERROR));
+    return(os2err(DosClose((HFILE) opaque)) == NO_ERROR);
 } /* __PHYSFS_platformClose */
 
 
@@ -677,7 +619,7 @@ int __PHYSFS_platformDelete(const char *path)
     if (__PHYSFS_platformIsDirectory(path))
         return(os2err(DosDeleteDir(path)) == NO_ERROR);
 
-    return(os2err(DosDelete(path) == NO_ERROR));
+    return(os2err(DosDelete(path)) == NO_ERROR);
 } /* __PHYSFS_platformDelete */
 
 
@@ -707,13 +649,6 @@ PHYSFS_sint64 __PHYSFS_platformGetLastModTime(const char *fname)
 } /* __PHYSFS_platformGetLastModTime */
 
 
-/* Much like my college days, try to sleep for 10 milliseconds at a time... */
-void __PHYSFS_platformTimeslice(void)
-{
-    DosSleep(10);
-} /* __PHYSFS_platformTimeslice(void) */
-
-
 PHYSFS_uint64 __PHYSFS_platformGetThreadID(void)
 {
     PTIB ptib;
@@ -754,7 +689,14 @@ void __PHYSFS_platformReleaseMutex(void *mutex)
     DosReleaseMutexSem((HMTX) mutex);
 } /* __PHYSFS_platformReleaseMutex */
 
-#endif  /* defined OS2 */
+
+/* !!! FIXME: Don't use C runtime for allocators? */
+int __PHYSFS_platformSetDefaultAllocator(PHYSFS_Allocator *a)
+{
+    return(0);  /* just use malloc() and friends. */
+} /* __PHYSFS_platformSetDefaultAllocator */
+
+#endif  /* PHYSFS_PLATFORM_OS2 */
 
 /* end of os2.c ... */
 
