@@ -1,15 +1,11 @@
 /**
  * Test program for PhysicsFS. May only work on Unix.
  *
- * Please see the file LICENSE in the source's root directory.
+ * Please see the file LICENSE.txt in the source's root directory.
  *
  *  This file written by Ryan C. Gordon.
  */
 
-#if HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -29,9 +25,9 @@
 
 #include "physfs.h"
 
-#define TEST_VERSION_MAJOR  1
+#define TEST_VERSION_MAJOR  2
 #define TEST_VERSION_MINOR  0
-#define TEST_VERSION_PATCH  0
+#define TEST_VERSION_PATCH  2
 
 static FILE *history_file = NULL;
 static PHYSFS_uint32 do_buffer_size = 0;
@@ -132,6 +128,59 @@ static int cmd_addarchive(char *args)
 } /* cmd_addarchive */
 
 
+static int cmd_mount(char *args)
+{
+    char *ptr;
+    char *mntpoint = NULL;
+    int appending = 0;
+
+    if (*args == '\"')
+    {
+        args++;
+        ptr = strchr(args, '\"');
+        if (ptr == NULL)
+        {
+            printf("missing string terminator in argument.\n");
+            return(1);
+        } /* if */
+        *(ptr) = '\0';
+    } /* if */
+    else
+    {
+        ptr = strchr(args, ' ');
+        *ptr = '\0';
+    } /* else */
+
+    mntpoint = ptr + 1;
+    if (*mntpoint == '\"')
+    {
+        mntpoint++;
+        ptr = strchr(mntpoint, '\"');
+        if (ptr == NULL)
+        {
+            printf("missing string terminator in argument.\n");
+            return(1);
+        } /* if */
+        *(ptr) = '\0';
+    } /* if */
+    else
+    {
+        ptr = strchr(mntpoint, ' ');
+        *(ptr) = '\0';
+    } /* else */
+    appending = atoi(ptr + 1);
+
+    /*printf("[%s], [%s], [%d]\n", args, mntpoint, appending);*/
+
+    if (PHYSFS_mount(args, mntpoint, appending))
+        printf("Successful.\n");
+    else
+        printf("Failure. reason: %s.\n", PHYSFS_getLastError());
+
+    return(1);
+} /* cmd_mount */
+
+
 static int cmd_removearchive(char *args)
 {
     if (*args == '\"')
@@ -328,7 +377,7 @@ static int cmd_stressbuffer(char *args)
         printf("buffer must be greater than or equal to zero.\n");
     else
     {
-        PHYSFS_file *f;
+        PHYSFS_File *f;
         int rndnum;
 
         printf("Stress testing with (%d) byte buffer...\n", num);
@@ -645,7 +694,7 @@ static int cmd_issymlink(char *args)
 
 static int cmd_cat(char *args)
 {
-    PHYSFS_file *f;
+    PHYSFS_File *f;
 
     if (*args == '\"')
     {
@@ -699,7 +748,7 @@ static int cmd_cat(char *args)
 
 static int cmd_filelength(char *args)
 {
-    PHYSFS_file *f;
+    PHYSFS_File *f;
 
     if (*args == '\"')
     {
@@ -729,7 +778,7 @@ static int cmd_filelength(char *args)
 
 static int cmd_append(char *args)
 {
-    PHYSFS_file *f;
+    PHYSFS_File *f;
 
     if (*args == '\"')
     {
@@ -777,7 +826,7 @@ static int cmd_append(char *args)
 
 static int cmd_write(char *args)
 {
-    PHYSFS_file *f;
+    PHYSFS_File *f;
 
     if (*args == '\"')
     {
@@ -888,6 +937,7 @@ static const command_info commands[] =
     { "init",           cmd_init,           1, "<argv0>"                    },
     { "deinit",         cmd_deinit,         0, NULL                         },
     { "addarchive",     cmd_addarchive,     2, "<archiveLocation> <append>" },
+    { "mount",          cmd_mount,          3, "<archiveLocation> <mntpoint> <append>" },
     { "removearchive",  cmd_removearchive,  1, "<archiveLocation>"          },
     { "enumerate",      cmd_enumerate,      1, "<dirToEnumerate>"           },
     { "ls",             cmd_enumerate,      1, "<dirToEnumerate>"           },
@@ -990,7 +1040,7 @@ static int process_command(char *complete_cmd)
         return(0);
     } /* if */
 
-    cmd_copy = malloc(strlen(complete_cmd) + 1);
+    cmd_copy = (char *) malloc(strlen(complete_cmd) + 1);
     if (cmd_copy == NULL)
     {
         printf("\n\n\nOUT OF MEMORY!\n\n\n");
@@ -1066,7 +1116,9 @@ static void open_history_file(void)
 
         do
         {
-            fgets(buf, sizeof (buf), f);
+            if (fgets(buf, sizeof (buf), f) == NULL)
+                break;
+
             if (buf[strlen(buf) - 1] == '\n')
                 buf[strlen(buf) - 1] = '\0';
             add_history(buf);
@@ -1121,7 +1173,7 @@ int main(int argc, char **argv)
         buf = readline("> ");
 #else
         int i;
-        buf = malloc(512);
+        buf = (char *) malloc(512);
         memset(buf, '\0', 512);
         printf("> ");
         for (i = 0; i < 511; i++)
